-- Groups and Bars management (C) 2011 Michael

local Addon= BurnIt

local GetTime = GetTime
local fmt = string.format
local tinsert= table.insert
local tremove= table.remove
local pairs= pairs
local ipairs= ipairs
local unpack= unpack
local next= next
local tsort= table.sort
local floor= math.floor


--{{{ Time tracker management
local TimeTrackerAddBar, TimeTrackerRemoveBar
do
	local frame
	local trackedBars={}
	local count= 0
	local elapsedTime= 0

	local function Timer(_, elapsed)
		elapsedTime= elapsedTime+ elapsed
		if elapsedTime>0.1 then
			local CurTime= GetTime()
			for bar in pairs(trackedBars) do
				bar:Update(CurTime)
			end
			elapsedTime= 0
		end
	end
	
	TimeTrackerAddBar= function(bar)
		frame= CreateFrame("Frame")
		frame:SetScript("OnUpdate", Timer)
		TimeTrackerAddBar= function(bar) 
			if not trackedBars[bar] then
				trackedBars[bar]= true 
				count= count+1
				if count<2 then frame:Show() end
				--		
				local parent= bar.parent
				parent.count= parent.count + 1 
				if parent.count<2 and not parent:IsShown() then
					parent:Show()
				end
			end	
		end
		TimeTrackerAddBar(bar)
	end	
	
	TimeTrackerRemoveBar= function(bar)
		if trackedBars[bar] then
			trackedBars[bar]= nil
			count= count-1
			if count<1 then	frame:Hide() end
			--
			local parent= bar.parent
			parent.count= parent.count - 1 
			if parent.count<1 and parent.queueHide then
				parent.queueHide= nil
				parent:Hide()
			end
		end	
	end
end
-- Time tracker end }}}


--{{{ Bar Groups management start
local CreateGroup, Groups
do	
	Groups= {}						
	-- Published methods: AddBar, Layout, AddBar,DelBar,ThemeBar, Theme
	local AddBar,DelBar,ThemeBar,Layout,Sort, Reorder, Theme, OnBarHide,OnBarShow
	
	Layout= function(self, w,h)
		local w,h= self:GetWidth(), self.dbx.bar.height
		local allBars= { self:GetChildren() }
		for i=1,#allBars do
			local bar= allBars[i]
			bar:SetSize(w,h)
			bar:Layout()
		end
		Sort(self)
		Reorder(self)
	end
	
	Sort= function(self)
		tsort( self.Bars, function(a,b) return (a.dbx.position<b.dbx.position) end )
	end
	
	Theme= function(self)
		local bars= { self:GetChildren() }
		for i=1,#bars do
			ThemeBar(self,bars[i])
		end
	end
	
	Reorder= function(self, index)
		local bars= self.Bars
		index= index or 1	
		local prevBar= index>1 and bars[index-1] or self
		for i=index,#bars do
			local bar= bars[i]
			bar.index= i
			bar:SetPoint("TOPLEFT", prevBar, "BOTTOMLEFT")
			prevBar= bar
		end
	end

	OnBarShow= function(bar)
		local self= bar.parent
		local bars= self.Bars
		local index= #bars+1
		local position= bar.dbx.position
		if position then
			if position<=1 then
				index=1
			elseif position<index then
				index= floor(position)
			end	
		end
		tinsert(bars,index,bar)
		bar.index= index
		Reorder(self,index)
	end
	
	OnBarHide= function(bar)
		local self= bar.parent
		local index= bar.index
		tremove(self.Bars,index)
		Reorder(self,index)
		bar.index= nil
	end

	ThemeBar= function(self,bar)
		local function setting(bar,key)
			return bar.dbx[key] or self.dbx.bar[key]
		end

		local font     = setting(bar, "font")
		local fontSize = setting(bar, "fontSize")
		local fontColor= setting(bar, "fontColor")
		local texture  = setting(bar, "texture")
		local color    = setting(bar, "color")
		local bgColor  = setting(bar, "bgColor")
		local bgTexture= setting(bar, "bgTexture")

		bar:SetBackdrop({
			 bgFile = bgTexture, tile = false, tileSize = 16,
			 edgeFile = nil, edgeSize = 0,
			 insets = {left = 0, right = 0, top = 0, bottom = 0},
		 })
		bar:SetBackdropColor( unpack(bgColor) )

		bar.labelTime:SetFont(font,fontSize)
		bar.labelTime:SetTextColor( unpack(fontColor) )
		
		bar.labelText:SetFont(font,fontSize)
		bar.labelText:SetTextColor( unpack(fontColor) )
					
		bar.barTime:SetStatusBarTexture(texture)
		bar.barTime:SetStatusBarColor( unpack(color) )

		if bar.stackText then
			bar.stackText:SetTextColor( unpack(fontColor) )
			bar.stackText:SetFont(font, fontSize+2, "OUTLINE" )
		end
		
	end
	
	AddBar= function(self, bar)
		bar:SetParent(self)
		bar.parent= self
		bar:SetScript("OnShow", OnBarShow)
		bar:SetScript("OnHide", OnBarHide)
		if bar:IsShown() then
			local bars= self.Bars
			bars[#bars+1]= bar
			bar.index= #bars
		end
		bar:SetShow(bar.dbx.visible)
		return bar
	end

	DelBar= function(self, bar)
		bar:Hide()
		bar:SetScript("OnShow", nil)
		bar:SetScript("OnHide", nil)
		bar:SetParent(nil)
		bar.parent=nil
		bar:ClearAllPoints()
		return bar
	end

	UpdateVisibility= function(self, inCombat)
		if inCombat==nil then inCombat= InCombatLockdown() end
		if self.hidden or self.dbx.hidden then
			self:Hide()
		elseif self.dbx.hiddenOOC and (not inCombat) then
     		if self.count>0 then
				self.queueHide= true
			else	
				self:Hide()
			end	
		else
			self:Show()
		end	
	end
	
	CreateGroup= function(setup, frame)
		local group= frame or CreateFrame("Frame",nil, UIParent)
		group.count= 0
		group.Bars= {}
		group.dbx= setup
		group:SetWidth(setup.bar.width)
		group:SetHeight(12)
		group.AddBar= AddBar
		group.DelBar= DelBar
		group.ThemeBar= ThemeBar
		group.Sort= Sort
		group.Reorder= Reorder
		group.Layout= Layout
		group.Theme= Theme
		group.UpdateVisibility= UpdateVisibility
		tinsert(Groups,group)
		group.index= #Groups
		group:ClearAllPoints()
		group:SetScale(setup.scale or 1)
		group:SetPoint(setup.point or "CENTER", UIParent, setup.relPoint or "CENTER", setup.offX or 0, setup.offY or 0)
		return group
	end
end	
-- Bar groups end }}}


--{{{ Bars management start
local CreateBar,Bars
do	
	Bars= {}
	-- Published methods: Layout, Update, SetText, SetTime, SetIcon, Enable, Disable
	local Layout,Update,SetText,SetTime,SetStack,SetIcon,SetShow,FadeIn,FadeOut,Show,Hide, AddSpark, AddStack

	AddSpark = function(self, timestamp)
		self.sparks = self.sparks or {}
		local spark= self.barTime:CreateTexture(nil, "OVERLAY" )
		spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
		spark:SetVertexColor(1, 1, 0)
		spark:SetBlendMode("ADD")
		spark:SetWidth( 6 )
		spark.timestamp = timestamp
		tinsert( self.sparks, spark )
	end
	
	AddStack = function(self)
		self.stackText= self:CreateFontString(nil, "OVERLAY", "ChatFontNormal")
		self.stackText:SetJustifyH("MIDDLE")
		self.stackText:SetJustifyV("CENTER")
		self.stackText:Show()
	end
	
	FadeIn = function(self)
		self:SetAlpha(1)
		self:Update()
		TimeTrackerAddBar(self)
		self.enabled= true
	end

	FadeOut = function(self)
		TimeTrackerRemoveBar(self)
		self.labelTime:SetText("")
		self.labelText:SetText(self.defaultText)
		self.barTime:SetValue(0)
		self:SetAlpha(0.45)
		self.enabled= false
		if self.stackText then
			self.stackText:SetText("")
		end
		if self.sparks then
			for _,spark in next, self.sparks do
				spark:Hide()
			end
		end
	end
	
	Show= function(self)
		self:Update()
		TimeTrackerAddBar(self)
		self:Show()
		self.enabled= true
	end
	
	Hide= function(self)
		TimeTrackerRemoveBar(self)
		self:Hide()
		self.enabled= false
	end
	
	Layout= function(self)
		local w,h= self:GetSize()
		
		self.icon:ClearAllPoints()
		self.icon:SetSize(h,h)
		self.icon:SetPoint("LEFT", self, "LEFT", 0, 0)

		self.barTime:ClearAllPoints()
		self.barTime:SetPoint("LEFT", self.icon, "RIGHT")
		self.barTime:SetPoint("RIGHT", self)
		self.barTime:SetPoint("TOP", self)
		self.barTime:SetPoint("BOTTOM", self)
		
		self.labelText:ClearAllPoints()
		self.labelText:SetSize((w-h)*0.85,h)
		self.labelText:SetPoint("LEFT", self.icon, "RIGHT", 1, 0)

		self.labelTime:ClearAllPoints()
		self.labelTime:SetSize((w-h)*0.35,h)
		self.labelTime:SetPoint("RIGHT", self, "RIGHT", -1, 0)	
	
		if self.sparks then
			for _,spark in next, self.sparks do
				spark:SetHeight( h*1.5 )
			end
		end
		
		if self.stackText then
			self.stackText:ClearAllPoints()
			self.stackText:SetPoint("LEFT"  , self.icon, "LEFT"  , -h,   0)
			self.stackText:SetPoint("RIGHT" , self.icon, "RIGHT" ,  h+2, 0)
			self.stackText:SetPoint("TOP"   , self.icon, "TOP", 0, 0)
			self.stackText:SetPoint("BOTTOM", self.icon)
		end
		
	end
	
	SetIcon= function(self, texture)
		self.icon:SetTexture(texture)
	end
	
	SetText= function(self, text)
		self.labelText:SetText(text)	
	end
	
	SetTime= function(self, expiration, duration)
		self.expiration= expiration
		self.duration  = duration
	end

	SetStack = function(self,count)
		self.stackCount = (count and count>1) and count or nil
	end
	
	SetShow= function(self, alwaysVisible)
		if alwaysVisible then
			self.Enable= FadeIn
			self.Disable= FadeOut
			self:SetAlpha(0.45)
			self:Show()
		else
			self.Enable= Show
			self.Disable= Hide
			self:SetAlpha(1)
			self:Hide()
		end		
	end
	
	Update= function(self, CurTime)
		local remain= self.expiration - (CurTime or GetTime())
		if remain>0 then
			self.labelTime:SetText( fmt(remain<5 and "%.1f" or "%d",remain) ) 
			self.barTime:SetValue( remain / self.duration )
			if self.sparks then
				local width = self.barTime:GetWidth()
				local mul   = width / self.duration
				for _,spark in next, self.sparks do
					local pos= spark.timestamp * mul
					if pos<width then
						spark:Show()
						spark:SetPoint("CENTER", self.barTime, "LEFT", pos , 0 )
					else
						spark:Hide()
					end					
				end
			end
			if self.stackText then
				self.stackText:SetText( self.stackCount or "")
			end			
		else
			self:Disable()
		end
	end
	
	CreateBar = function(text, icon)
		local self= CreateFrame("Frame")
	
		self.description= self.description or text
	
		self:SetBackdrop({
					 bgFile = "Interface\\Addons\\BurnIt\\media\\gradient32x32", tile = false, tileSize = 16,
					 edgeFile = nil, edgeSize = 0,
					 insets = {left = 0, right = 0, top = 0, bottom = 0},
				 })
		self:SetBackdropColor(0.3, 0.3, 0.3, 0.75)

		self.defaultText= text
		
		self.icon= self:CreateTexture(nil, "ARTWORK")
		self.icon:SetTexture(icon or DEFAULT_ICON)

		self.barTime= CreateFrame("StatusBar", nil, self)
		self.barTime:SetMinMaxValues(0,1)
		self.barTime:SetValue(0)
		self.barTime:SetStatusBarTexture("Interface\\Addons\\BurnIt\\media\\gradient32x32")
		self.barTime:SetStatusBarColor(1,0,0,1)
		self.barTime:SetFrameLevel( self:GetFrameLevel()+1 )
		
		self.labelText= self.barTime:CreateFontString(nil, "ARTWORK", "ChatFontNormal")
		self.labelText:SetJustifyH("LEFT")
		self.labelText:SetText(text)
		
		self.labelTime= self.barTime:CreateFontString(nil, "ARTWORK", "ChatFontNormal")
		self.labelTime:SetJustifyH("RIGHT")
	
		self.expiration= 0
		self.duration= 0
		
		self.Layout   = Layout
		self.Update   = Update
		self.SetText  = SetText
		self.SetTime  = SetTime
		self.SetStack = SetStack
		self.SetIcon  = SetIcon
		self.SetShow  = SetShow
		self.Enable   = Show
		self.Disable  = Hide
		self.AddSpark = AddSpark
		self.AddStack = AddStack

		Bars[#Bars+1]= self
		
		return self
	end
	
end
-- Bars end }}}

--{{{{ Publishing Groups & Bars methods and variables

Addon.Groups= Groups
Addon.Bars= Bars
Addon.CreateGroup= CreateGroup
Addon.CreateBar= CreateBar

--}}}}