-- CombatLog and other events tracking (C) 2011 Michael

local Addon= BurnIt

local GetTime= GetTime
local UnitAura= UnitAura
local UnitGUID= UnitGUID
local fmt= string.format
local bit_band= bit.band
local pairs, tinsert, tremove, tostring, select = pairs, tinsert, tremove, tostring, select

--{{{{ Aura Tracking
do
	local AuraTrackerAddAura, AuraTrackerEnable, AuraTrackerDisable
	local Auras= {}
	local Events= {}
	local CombatActions= {}
	local DamageHandlers= {}
	local function Dummy() end
	
	AuraTrackerEnable= function()
		local Frame= Addon.MainFrame
		Frame:SetScript("OnEvent", function(self, event, ...) Events[event](self, ... ) end)
		Frame:RegisterEvent("PLAYER_REGEN_ENABLED")
		Frame:RegisterEvent("PLAYER_REGEN_DISABLED")
		Frame:RegisterEvent("PLAYER_TARGET_CHANGED")
		Frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		Frame:RegisterEvent("PLAYER_TALENT_UPDATE")
	end	
	
	AuraTrackerDisable= function()
		local Frame= Addon.MainFrame
		Frame:UnregisterEvent("PLAYER_REGEN_ENABLED")
		Frame:UnregisterEvent("PLAYER_REGEN_DISABLED")
		Frame:UnregisterEvent("PLAYER_TARGET_CHANGED")
		Frame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		Frame:UnregisterEvent("PLAYER_TALENT_UPDATE")
		Frame:SetScript("OnEvent", nil)
	end
	
	AuraTrackerAddAura= function(handler, setup)
	
		local auraType= setup.auraType
		local unitId= setup.destUnit
		local spells= setup.spells
		local castedByPlayer= setup.castByMe
	
		-- Fill noexistant handler methods
		if not handler.Update then handler.Update= Dummy end
		if not handler.Add then handler.Add= handler.Update end
		if not handler.Remove then handler.Remove= handler.Update end
		if not handler.Dots then handler.Dots= Dummy end
		local auraMask,sourceGUID,destGUID,destFlags
		if auraType then
			auraMask= (auraType=="BUFF" and (castedByPlayer and "PLAYER HELPFUL" or "HELPFUL")) or
					  (auraType=="DEBUFF" and (castedByPlayer and "PLAYER HARMFUL" or "HARMFUL")) 
		end
		
		if unitId then
			if unitId=="player" then
				destGUID= UnitGUID("player")
			else
				destFlags= (unitId=="target" and COMBATLOG_OBJECT_TARGET) or
						   (unitId=="focus"  and COMBATLOG_OBJECT_FOCUS) 
			end					
		end					
		if castedByPlayer then
			sourceGUID= UnitGUID("player")
		end
		--
		local aura= {
			handler= handler,
			unitId= unitId,
			castedByPlayer= castedByPlayer,
			sourceGUID= sourceGUID,
			destFlags= destFlags,
			destGUID= destGUID,
			auraType= auraType,
			auraMask= auraMask,
		}
		for _,spellId in ipairs(spells) do
			Auras[spellId]= aura
		end
		if handler.Damage then 
			tinsert( DamageHandlers, aura)
		end
	end
	
	local function AuraApplied(aura, ...)
		aura.handler:Add(aura, ...)
	end
	
	local function AuraRemoved(aura, ...)
		aura.handler:Remove(aura, ...)
	end

	local function AuraDamage(aura, ...)
		aura.handler:Dots( aura, select(15,...), select(21,...), ... )
	end

	local function SpellDamage(aura, ...)
		aura.handler:Damage( aura, ...)
	end
	
	CombatActions.SPELL_AURA_APPLIED= AuraApplied
	CombatActions.SPELL_AURA_REFRESH= AuraApplied
	CombatActions.SPELL_AURA_REMOVED= AuraRemoved
	CombatActions.SPELL_AURA_BROKEN= AuraRemoved
	CombatActions.SPELL_AURA_BROKEN_SPELL= AuraRemoved
	CombatActions.SPELL_PERIODIC_DAMAGE= AuraDamage
	CombatActions.SPELL_AURA_APPLIED_DOSE= AuraApplied
	CombatActions.SPELL_AURA_REMOVED_DOSE= AuraApplied
	
	---timestamp(1), event(2), hideCaster(3),  sourceGUID(4), sourceName(5), sourceFlags(6), destGUID(7), destName(8), destFlags(9), spellId(10), 
	-- spellName(11), spellSchool(12), amount(13), overkill(14),school(15), resisted(16), blocked(17), absorved(18), critical(19)
	local function ExecuteAction(aura, action, ...)
		local sourceGUID= aura.sourceGUID
		if not sourceGUID or sourceGUID==select(4,...) then
			local destGUID= aura.destGUID
			if not destGUID or destGUID==select(8,...) then
				local destFlags= aura.destFlags
				if not destFlags or bit_band(select(10,...),destFlags)==destFlags then
					action(aura, ...) 
				end
			end
		end
	end
	
	function Events:COMBAT_LOG_EVENT_UNFILTERED(...)
		--if select(2,...)=="SPELL_AURA_APPLIED" then
		--	print("->", select(2,...), select(10,...), select(11,...) )
		--end
		local event  = select(2,...)
		local action = CombatActions[ event ]
		if action then 
			local aura= Auras[ select(12,...) ]
			if aura then 
				ExecuteAction(aura, action, ...)
			end
		elseif event=="SPELL_DAMAGE" then
			for i=#DamageHandlers,1,-1 do
				SpellDamage( DamageHandlers[i], ... )
			end
		end
	end	

	do
		local updated= {}
		function Events:PLAYER_TARGET_CHANGED()
			for _,aura in pairs(Auras) do
				local handler,unit= aura.handler,aura.unitId
				if (not updated[handler]) and ((not unit) or unit=="target") then
					handler:Update(aura)
					updated[handler]= true
				end
			end
			wipe(updated)
		end
	end	
	
	function Events:PLAYER_REGEN_ENABLED()	
		local Groups= Addon.Groups
		for i=1,#Groups do
			Groups[i]:UpdateVisibility(false)
		end
	end
	
	function Events:PLAYER_REGEN_DISABLED()
		local Groups= Addon.Groups
		for i=1,#Groups do
			Groups[i]:UpdateVisibility(true)
		end
	end

	function Events:PLAYER_TALENT_UPDATE()
		if IsSpellKnown(11129) then
			Addon.MageSpec = "fire"
		elseif IsSpellKnown(12042) then
			Addon.MageSpec = "arcane"
		elseif IsSpellKnown(71757) then
			Addon.MageSpec = "frost"
		else 
			Addon.MageSpec = "unknow"
			return
		end
		local Groups= Addon.Groups
		for i=1,#Groups do
			local group = Groups[i]
			group.hidden = group.dbx.MageSpec and group.dbx.MageSpec~=Addon.MageSpec
			group:UpdateVisibility()
		end
	end

	--{{{{ Publishing Tracker methods
	Addon.AuraTrackerAddAura = AuraTrackerAddAura
	Addon.AuraTrackerEnable  = AuraTrackerEnable
	Addon.AuraTrackerDisable = AuraTrackerDisable
	Addon.UpdateTalents      = Events.PLAYER_TALENT_UPDATE
	--}}}}

end
--}}}}

