--[[
Copyright (C) 2011 Michael

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
]]

-- Reference to BurnIt addon
local BurnIt= BurnIt

-- Localization table
local L= BurnIt.Locale

-- Our BurnItOptions Addon table
BurnItOptions= {}
local Addon= BurnItOptions

-- LibSharedMedia reference
local Media

-- Group frames recycling
local unusedFrames= {}

-- Default settings for new groups
Addon.groupDB={
	point = "CENTER", relPoint = "CENTER",	offX= 0, offY= 0, scale=1, hiddenOOC= true,
	bar= {
		width= 140,
		height= 16,
		font= STANDARD_TEXT_FONT,
		fontSize = 10,	
		fontColor= {1,1,1,1},
		color= { 1,0,0,1 },
		texture="Interface\\Addons\\BurnIt\\media\\gradient32x32",
		bgColor= { 0.3, 0.3, 0.3, 0.75 },
		bgTexture= "Interface\\Addons\\BurnIt\\media\\gradient32x32",
	}
}

--{{ Miscellaneous functions
local function copyTable(s)
	local d = {}
	for k,v in pairs(s) do
		if type(v) == "table" then
			v = copyTable(v)
		end
		d[k] = v
	end
	return d
end
--}}

--{{{{ Main stuff
function Addon:Initialize()
	Media= LibStub("LibSharedMedia-3.0", true)
	if Media then
		Media:Register("statusbar", "BurnIt Gradient", "Interface\\AddOns\\BurnIt\\media\\gradient32x32.tga")
	end
	self.menuFrame= CreateFrame("Frame", "BurnItMenu", UIParent, "UIDropDownMenuTemplate") 
end

function Addon:UnlockFrames()
	BurnIt.AuraTrackerDisable()
	for g=1,#BurnIt.Groups do
		self:UnlockGroup( BurnIt.Groups[g] )
	end
	Addon.Unlocked= true
end

function Addon:LockFrames()
	for g=1,#BurnIt.Groups do
		self:LockGroup( BurnIt.Groups[g] )
	end
	BurnIt.AuraTrackerEnable()
	BurnIt:UpdateTalents()
	Addon.Unlocked= false
end

function Addon:UnlockGroup(group)
	group:SetBackdrop({
		bgFile = "Interface\\Addons\\BurnIt\\media\\gradient32x32.tga",
		edgeFile = "Interface\\Addons\\BurnIt\\media\\gradient32x32.tga",  
		tile = false, tileSize = 16, edgeSize = 1, 
		insets = { left = 1, right = 1, top = 1, bottom = 1 },
	 })
	group:SetBackdropBorderColor(0,0,0,0.9)
	group:SetBackdropColor(0,0,0,0.9);
	if not group.labelTitle then
		group.labelTitle= group:CreateFontString(nil, "OVERLAY", "ChatFontNormal")
		group.labelTitle:SetJustifyH("CENTER")
		group.labelTitle:SetFont(STANDARD_TEXT_FONT,9)
		group.labelTitle:SetAllPoints()
	end
	group.labelTitle:SetText(L["Group"] .." "..(group.index))

	if not group.resizeFrame then
		group.resizeFrame= CreateFrame("Frame", nil, UIParent)
		group.resizeFrame:SetBackdrop({
			bgFile = "Interface\\Addons\\BurnItOptions\\media\\hresize.tga",
			tile = false, tileSize = 16, edgeSize = 0, 
			insets = { left = 0, right = 0, top = 0, bottom = 0 },
		 })
		group.resizeFrame:SetBackdropColor(0.9,0.9,0.9,1);
		group.resizeFrame:SetAlpha(1)
		group.resizeFrame:SetPoint("RIGHT", group, "RIGHT")
		group.resizeFrame:SetPoint("TOP", group, "TOP")
		group.resizeFrame:SetPoint("BOTTOM", group, "BOTTOM")
		group.resizeFrame:SetWidth( group:GetHeight() )
		group.resizeFrame:EnableMouse(true)
		group.resizeFrame:SetScript("OnMouseDown", Addon.GroupResizeOnMouseDown)
		group.resizeFrame:SetScript("OnMouseUp", Addon.GroupResizeOnMouseUp)
		group.resizeFrame:SetScript("OnEnter", function() SetCursor("ITEM_CURSOR") end )
		group.resizeFrame.group= group
	end
	group.resizeFrame:SetFrameLevel( group:GetFrameLevel()+1 )
	group.resizeFrame:Show()

	group:SetScript("OnMouseDown", Addon.GroupOnMouseDown)
	group:SetScript("OnMouseUp", Addon.GroupOnMouseUp)
	group:SetScript("OnEnter", function() SetCursor("Interface\\Cursor\\EngineerSkin.blp") end )
	
	group:SetMovable(true)
	group:SetResizable(true)
	group:EnableMouse(true)

	local time= GetTime()
	local bars= { group:GetChildren() }
	for i=1,#bars do
		local bar= bars[i]
		bar:SetTime( time+math.random(10,30), 30)
		bar:SetText( bar.description )
		bar:SetAlpha(1)
		bar:Update()
		bar:Show()
		bar:SetMovable(true)
		bar:EnableMouse(true)
		bar:SetScript("OnMouseDown", Addon.BarOnMouseDown)
		bar:SetScript("OnMouseUp", Addon.BarOnMouseUp)
		bar:SetScript("OnEnter", function() SetCursor("CAST_CURSOR") end )
	end
	group:Show()
	self:Layout(group)
end

function Addon:LockGroup(group)
	group:SetBackdrop(nil)
	group:SetBackdropBorderColor(0,0,0,0)
	group:SetBackdropColor(0,0,0,0)
	group.labelTitle:SetText("")
	group:SetScript("OnMouseDown", nil)
	group:SetScript("OnMouseUp", nil)
	group:SetScript("OnEnter", nil)
	group:SetMovable(false)
	group:EnableMouse(false)
	group.resizeFrame:Hide()
	local bars= { group:GetChildren() }
	for i=1,#bars do
		local bar= bars[i]
		bar:SetMovable(false)
		bar:EnableMouse(false)
		bar:SetScript("OnMouseDown", nil)
		bar:SetScript("OnMouseUp", nil)
		bar:SetScript("OnEnter", nil)
		bar:SetTime(0,0)
		bar:SetText(bar.defaultText)
		bar:Disable()
	end
	group:Reorder()
	group:UpdateVisibility()
end

function Addon:Layout(group)
	group:Sort()
	local offy, w, h= 0, group:GetWidth(), group.dbx.bar.height
	for i=1,#group.Bars do
		local bar= group.Bars[i]
		bar:SetSize(w,h)
		bar:Layout()
		bar.index= i
		bar:ClearAllPoints()
		bar:SetPoint("TOPLEFT", group, "BOTTOMLEFT", 0, offy )
		offy= offy - bar:GetHeight()
		if bar.dbx.position>1 and bar.dbx.position<100 then
			bar.dbx.position= i
		end
	end
end
--}}}}

--{{{{ Groups sizing and moving
function Addon:GroupResizeOnMouseDown(button)
	local frame
	self,frame= Addon,self
	if button=="LeftButton" then
		self.isSizingGroup= true
		frame.group:StartSizing("RIGHT") 
	end
end

function Addon:GroupResizeOnMouseUp(button)
	local frame
	self,frame= Addon,self
	if self.isSizingGroup then
		self.isSizingGroup= false
		frame.group:StopMovingOrSizing()
		frame.group.dbx.bar.width= frame.group:GetWidth()
		self:Layout(frame.group)
	end
end

function Addon:GroupOnMouseDown(button)
	local group
	self,group= Addon,self
	if button=="LeftButton" then
		group:StartMoving() 
		self.isMovingGroup= true
	elseif button=="RightButton" then
		self:OpenMenuGroup(group)
	end	
end

function Addon:GroupOnMouseUp()
	local group
	self,group= Addon,self
	if self.isMovingGroup then
		self.isMovingGroup= false
		group:StopMovingOrSizing()
		local dbx= group.dbx
		dbx.point, _, dbx.relPoint, dbx.offX, dbx.offY = group:GetPoint()
	end	
end
--}}}}

--{{{{ Bars dragging
function Addon:GetGroupBarFromBar(sourceBar)
	
		local function PointInsideFrame(f,x,y)
		    return f:IsVisible() and x>=f:GetLeft() and x<=f:GetRight() and y>=f:GetBottom() and y<=f:GetTop()
		end

	local x,y= sourceBar:GetCenter()
	local rGroup
	for g=1,#BurnIt.Groups do
		local group= BurnIt.Groups[g]
		for i=1,#group.Bars do
			local bar= group.Bars[i]
			if bar~=sourceBar and PointInsideFrame(bar,x,y) then
				if y-bar:GetBottom() < bar:GetTop()-y then
					return group, bar, 0.5
				else
					return group, bar, -0.5
				end
			end		
		end
		if PointInsideFrame(group,x,y) then rGroup= group end
	end		
	return rGroup
end

function Addon:BarOnMouseDown(button)
	local bar
	self,bar= Addon,self
	if button=="LeftButton" then
		bar:SetFrameStrata("DIALOG")
		bar:StartMoving() 
		self.isMovingBar= true
	elseif button=="RightButton" then
		self:OpenMenuBar(bar)
	end	
end

function Addon:BarOnMouseUp()
	local bar
	self,bar= Addon,self
	if self.isMovingBar then 
		self.isMovingBar= false
		bar:StopMovingOrSizing()
		bar:SetFrameStrata( bar.parent:GetFrameStrata() )
		local group= bar.parent
		local destGroup,destBar,destPos= self:GetGroupBarFromBar(bar)
		if destGroup then
			bar.dbx.position= destBar and (destBar.index + destPos) or 0
			if destGroup~=group then
				bar.dbx.group= destGroup.index
				group:DelBar(bar)
				destGroup:AddBar(bar)
				bar:SetAlpha(1)
				bar:Show()
			end
			self:Layout(destGroup)
		end
		self:Layout(group)
	end	
end
--}}}} 

--{{{{ Dropdown Menus: groups and bars

--{{ Shared funtions
function Addon:CreateNumbersMenu(func, from, to, step, count)
	step= step or 1
	count= count or 1000
	local page= step*count
	local menu,subMenu
	for i=0,to,page do
		subMenu= {}
		local j1,j2= math.max(from,i), math.min(to,i+page-step)
		for j=j1,j2,step do
			table.insert( subMenu, { text= j, value= j, func=func, notCheckable=true, keepShownOnClick= true } )
		end
		if to>=page then
			if not menu then menu={} end
			table.insert( menu, { text = j1.." - "..j2, notCheckable= true, hasArrow = true,  menuList = subMenu } )
		end	
	end
	return to>=page and menu or subMenu
end


function Addon:CreateMediaMenu(type, func)
	local menu, items= {}, Media and Media:List(type) or {}
	local subMenu= menu
	for i,v in pairs(items) do
		if i%10==0 and #subMenu>0 then
			if menu~=subMenu then
				menu[#menu].text= strsub(subMenu[1].text,1,1) .." - " .. strsub(subMenu[#subMenu].text,1,1)
			end	
			subMenu= {}
			table.insert( menu, { text = "",  notCheckable= true, hasArrow = true,  menuList = subMenu } )
		end
		table.insert( subMenu, { text=v, value=v, func=func, notCheckable=true, keepShownOnClick= true } )
	end
	if menu[#menu].text=="" then
		menu[#menu].text= strsub(subMenu[1].text,1,1) .." - " .. strsub(subMenu[#subMenu].text,1,1)
	end		
	return menu
end

function Addon:SetMenuOptionColor(option, color)
	option.r, option.g, option.b= unpack(color)
	option.opacity= 1 - color[4]
end
--}}

--{{ Bar dropdown menu
function Addon:OpenMenuBar(bar)
	local menu= self:GetMenuBar(bar)
	EasyMenu(menu, self.menuFrame, "cursor", 0 , 0, "MENU", 0)
end

function Addon:SetBarColor(bar)
	local color= bar.dbx.color
	color[1],color[2],color[3]= ColorPickerFrame:GetColorRGB() 
    color[4]= 1-OpacitySliderFrame:GetValue() 
	bar.parent:ThemeBar(bar)
end

function Addon:SetBarPosition(bar, position)
	bar.dbx.position= position 
	self:Layout(bar.parent)
end

function Addon:GetMenuBar(bar)
	self.menuBar= {
	     [1]= { text = L["Bar settings"],  notCheckable= true, isTitle = true },
	     [2]= { text = L["Color"],  notCheckable= true,  hasColorSwatch =true, hasOpacity= true,
				swatchFunc = function() self:SetBarColor(self.curBar) end, 
				opacityFunc= function() self:SetBarColor(self.curBar) end, 
			  },	
	     [3]= { text = L["Always visible"],  checked= function() return self.curBar.dbx.visible end, 
				func= function() 
					local bar= self.curBar
					bar.dbx.visible= not bar.dbx.visible  
					bar:SetShow(bar.dbx.visible)
					bar:SetAlpha(1)
					bar:Show()
				end 
			  },
	     [4]= { text = L["Position"],  notCheckable= true, hasArrow= true, menuList= {
					{ text=L["Here"],   checked= function() return self.curBar.dbx.position>1 and self.curBar.dbx.position<100 end,
									 func=    function() self:SetBarPosition(self.curBar, self.curBar.index) end, },
					{ text=L["Top"],    checked= function() return self.curBar.dbx.position==1   end, 
									 func   = function() self:SetBarPosition(self.curBar,1)   end,  },
					{ text=L["Bottom"], checked= function() return self.curBar.dbx.position==100 end, 
									 func   = function() self:SetBarPosition(self.curBar,100) end,  },	
			  } },
		 [5]= { text = L["Finish config"], notCheckable= true, func= function() self:LockFrames() end },		
	}	
	self.GetMenuBar= function(self,bar) 
		self.curBar= bar
		self.menuBar[1].text= bar.description or bar.spellName 	-- Bar Color title
		self:SetMenuOptionColor(self.menuBar[2], bar.dbx.color) -- Bar color menu item
		return self.menuBar 
	end 
	return self:GetMenuBar(bar)
end
--}}

--{{ Group dropdown menu
function Addon:OpenMenuGroup(group)
	local menu= self:GetMenuGroup(group)
	EasyMenu(menu, self.menuFrame, "cursor", 0 , 0, "MENU", 0)
end

function Addon:SetGroupBackgroundColor(group)
	local color= group.dbx.bar.bgColor
	color[1],color[2],color[3]= ColorPickerFrame:GetColorRGB() 
    color[4]= 1-OpacitySliderFrame:GetValue() 
	group:Theme()
end

function Addon:SetGroupFontColor(group)
	local color= group.dbx.bar.fontColor
	color[1],color[2],color[3]= ColorPickerFrame:GetColorRGB() 
    color[4]= 1-OpacitySliderFrame:GetValue() 
	group:Theme()
end

function Addon:SetGroupFontSize(group, value)
	group.dbx.bar.fontSize= value
	group:Theme()
end

function Addon:SetGroupFont(group, value)
	group.dbx.bar.font= Media:Fetch("font", value)
	group:Theme()
end

function Addon:SetGroupTexture(group,value)
	group.dbx.bar.texture= Media:Fetch("statusbar",value)
	group:Theme()
end

function Addon:SetGroupBarHeight(group,value)
	group.dbx.bar.height= value
	self:Layout(group)
end

function Addon:SetGroupBarTexture(group,value)
	group.dbx.bar.texture= Media:Fetch("statusbar",value)
	group:Theme()
end

function Addon:SetGroupBackgroundTexture(group,value)
	group.dbx.bar.bgTexture= Media:Fetch("statusbar",value)
	group:Theme()
end

function Addon:SetGroupScale(group,value)
	group.dbx.scale= value
	group:SetScale(value)
end

function Addon:SetGroupHidden(group, value)
	group.dbx.hidden= value
end

function Addon:SetGroupHiddenOOC(group, value)
	group.dbx.hiddenOOC= value
end

function Addon:SetGroupMageSpec(group, value)
	group.dbx.MageSpec= value
end

function Addon:GroupCanBeDeleted(group)
  local g= BurnIt.Groups
  return #g>1 and g[#g]==group and #group.Bars<1  
end

function Addon:CreateGroup()
	local GroupDb= BurnIt.db.Groups
	table.insert( GroupDb, copyTable(Addon.groupDB) )
	local group= BurnIt.CreateGroup( GroupDb[#GroupDb], table.remove(unusedFrames) )
	group:Theme()
	group:Layout()
	group:Show()
	self:UnlockGroup(group)
end

function Addon:DeleteGroup(group)
	if self:GroupCanBeDeleted(group) then
		self:LockGroup(group)
		group:Hide()
		table.remove( BurnIt.db.Groups )
		table.remove( BurnIt.Groups )
		table.insert(unusedFrames,group)
	else
		print(L["BurnIt warning: Only the last defined group can be deleted, and must be empty."])
	end
end

function Addon:GetMenuGroup(group)
	self.menuGroup= {
		[1]= { text = L["Bars settings"], notCheckable= true, isTitle = true},
		[2]= { text = L["Height"],   notCheckable= true, hasArrow = true, 
				menuList = self:CreateNumbersMenu(function(info) self:SetGroupBarHeight(self.curGroup,info.value) end, 1, 39,1,10 ) },
		[3]= { text = L["Texture"],   notCheckable= true, hasArrow = true, 
				menuList = self:CreateMediaMenu("statusbar", function(info) self:SetGroupBarTexture(self.curGroup,info.value) end) },
	    [4]= { text = L["Text Color"],  notCheckable= true,  hasColorSwatch =true, hasOpacity= true,
			    swatchFunc = function() self:SetGroupFontColor(self.curGroup) end, opacityFunc= function() self:SetGroupFontColor(self.curGroup) end, },
		[5]= { text = L["Text Size"], notCheckable= true, hasArrow = true, 
				menuList = self:CreateNumbersMenu(function(info) self:SetGroupFontSize(self.curGroup, info.value) end, 1,29,1,10 ) },
		[6]= { text = L["Text Font"],      notCheckable= true, hasArrow = true, 
				menuList = self:CreateMediaMenu("font", function(info) self:SetGroupFont(self.curGroup,info.value) end) },
	    [7]= { text = L["Background Color"],  notCheckable= true,  hasColorSwatch =true, hasOpacity= true,
			    swatchFunc = function() self:SetGroupBackgroundColor(self.curGroup) end, opacityFunc= function() self:SetGroupBackgroundColor(self.curGroup) end, },
		[8]= { text = L["Background Texture"],   notCheckable= true, hasArrow = true, 
				menuList = self:CreateMediaMenu("statusbar", function(info) self:SetGroupBackgroundTexture(self.curGroup,info.value) end) },
		[9]= { text = L["Group settings"], notCheckable= true, isTitle = true},
		[10]= { text = L["Visibility"],   notCheckable= true, hasArrow = true, 
				menuList = {
					{ text = L["Hidden Out of Combat"],  checked= function() return self.curGroup.dbx.hiddenOOC end, func= function(info) self:SetGroupHiddenOOC(self.curGroup, not self.curGroup.dbx.hiddenOOC) end },
					{ text = L["Always Hidden"],  checked= function() return self.curGroup.dbx.hidden end, func= function(info) self:SetGroupHidden(self.curGroup, not self.curGroup.dbx.hidden) end },
					{ text = L["Specialization"],   notCheckable= true, hasArrow = true, 
						menuList = {
							{ text = L["All"],  checked= function() return not self.curGroup.dbx.MageSpec end, func= function(info) self:SetGroupMageSpec(self.curGroup, nil) end },
							{ text = L["Fire"],  checked= function() return self.curGroup.dbx.MageSpec=="fire" end, func= function(info) self:SetGroupMageSpec(self.curGroup,"fire") end },
							{ text = L["Arcane"],  checked= function() return self.curGroup.dbx.MageSpec=="arcane" end, func= function(info) self:SetGroupMageSpec(self.curGroup,"arcane") end },
							{ text = L["Frost"],  checked= function() return self.curGroup.dbx.MageSpec=="frost" end, func=  function(info) self:SetGroupMageSpec(self.curGroup,"frost") end },
						},
					},	
				}},

	    [11]= { text = L["Scale"],   notCheckable= true, hasArrow = true, 
				menuList = self:CreateNumbersMenu(function(info) self:SetGroupScale(self.curGroup,info.value) end, 0.5, 1.6, 0.1 ) }, 
		[12]= { text = L["Create new group"], notCheckable= true, func= function() self:CreateGroup() end },		
		[13]= { text = L["Delete this group"], notCheckable= true, func= function() self:DeleteGroup(self.curGroup) end },		
		[14]= { text = L["Finish config"], notCheckable= true, func= function() self:LockFrames() end },		
	}	
	self.GetMenuGroup= function(self, group) 
		self.curGroup= group
		self:SetMenuOptionColor( self.menuGroup[4], group.dbx.bar.fontColor) -- Set text font item color
		self:SetMenuOptionColor( self.menuGroup[7], group.dbx.bar.bgColor) -- Set background menu item color
		return self.menuGroup 
	end 
	return self:GetMenuGroup(group)
end
--}}
--}}}}

--{{{{ ChatCommand processing
function Addon:OnChatCommand(msg)
	if Addon.Unlocked then
		Addon:LockFrames()
	else
		Addon:UnlockFrames()
	end	
end
--}}}}
